import json
import queue
import time

import threading
# from modbus_tk import modbus_tcp
from connector import Connector
from event_storage import EventStorage
import modbus_rtu_over_tcp_special
import modbus_tk.modbus_tcp as modbus_tcp
from log import OutPutLog

class ModbusMoxae1210Connector(Connector, threading.Thread):
    __master = 0
    _disConnectTime = 0

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__master = None
        self.__stopped = False
        self.__connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self._deviceID = config['deviceID']  # 设备地址
        self.__save_frequency = config['save_frequency']  # 数据存储时间间隔
        self.setDaemon(True)
        self.setName(name)
        self.__converter = converter
        self.__storager = EventStorage()
        self.__command_queue = queue.Queue(50)
        self.__last_save_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)
        self.__command = self.__storager.get_command_info(name)


    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        self.__connected = True
        command_list = self.__command
        while True:
            time.sleep(1)
            for i in range(len(command_list)):
                command_item = json.loads(command_list[i]['command'])   # {'length': 20, 'start_addr': 4096, 'output_value': [], 'function_code': 3}
                result = self.exec_command(command=command_item)
                if not result:
                    # 根据（轮询预计用时，剩余可用时间）计算重发次数
                    sent_times = 1
                    while sent_times < 5:
                        result = self.exec_command(command=command_item)
                        sent_times += 1
                        if result:
                            break
                else:
                    self.command_polling(result, resend_times=5)
            if self.__stopped:
                break

    def __connect(self):
        try:
            self.__master = modbus_tcp.TcpMaster(host=self._ip, port=self._port)
            # self.__master = modbus_rtu_over_tcp_special.RtuOverTcpMasterSpecial(host=self._ip, port=self._port)
        except Exception as e:
            self._log.error(e)

    def __reconnect(self):
        while True:
            try:
                self.__master = modbus_tcp.TcpMaster(host=self._ip)
                # self.__master = modbus_rtu_over_tcp_special.RtuOverTcpMasterSpecial(host=self._ip, port=self._port)
                print('client start connect to host/port:{}'.format(self._port))
                break
            except ConnectionRefusedError:
                print('modbus server refused or not started, reconnect to server in 5s .... host/port:{}'.format(
                    self._port))
                time.sleep(5)
            except Exception as e:
                print('do connect error:{}'.format(str(e)))
                time.sleep(5)

    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, content):
        pass

    def exec_command(self, command):
        """
        0x01: 读线圈寄存器
        0x02: 读离散输入寄存器
        0x03: 读保持寄存器
        0x04: 读输入寄存器
        0x05: 写单个线圈寄存器
        0x06: 写单个保持寄存器
        0x0f: 写多个线圈寄存器
        0x10: 写多个保持寄存器
        #supported modbus functions
        READ_COILS = 1
        READ_DISCRETE_INPUTS = 2
        READ_HOLDING_REGISTERS = 3
        READ_INPUT_REGISTERS = 4
        WRITE_SINGLE_COIL = 5
        WRITE_SINGLE_REGISTER = 6
        READ_EXCEPTION_STATUS = 7
        DIAGNOSTIC = 8
        REPORT_SLAVE_ID = 17
        WRITE_MULTIPLE_COILS = 15
        WRITE_MULTIPLE_REGISTERS = 16
        READ_WRITE_MULTIPLE_REGISTERS = 23
        DEVICE_INFO = 43
        """
        datadict = {}
        msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        function_code = int(command['function_code'])
        start_addr = int(command['start_addr'])
        length = int(command['length'])
        output_value = command['output_value']
        if function_code in (1, 2, 3, 4):
            # 读寄存器
            try:
                self.__master.set_timeout(0.5)
                self.__master.set_verbose(True)
                receive_data = self.__master.execute(self._deviceID, function_code, start_addr, length)
                for i in range(len(receive_data)):
                    addr = start_addr+i
                    datadict[addr] = receive_data[i]
                return datadict
            except Exception as e:
                self._log.error(e)
        elif function_code in (5, 6, 15, 16):
            # 写寄存器
            try:
                self.__master.set_timeout(1)
                self.__master.set_verbose(True)
                data = self.__master.execute(self._deviceID, function_code, start_addr, output_value=output_value)
                return data
            except Exception as e:
                self._log.error(e)
        else:
            self._log.debug(f'Unsupported function code.')

    def command_polling(self, result, resend_times=None):
        msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        if not self.__command_queue.empty():
            write_command = self.__command_queue.get()  # 写命令来自数组
            res = self.exec_command(write_command)
            if not res:
                # 根据（轮询预计用时，剩余可用时间）计算重发次数
                sent_times = 1
                while sent_times < resend_times:
                    res = self.exec_command(write_command)
                    sent_times += 1
                    if res:
                        break

        else:
            # 格式化数据
            format_data = self.__converter.convert(self.__data_point_config, result)

            if format_data:
                if format_data != "error" and format_data != 'pass':
                     # 往redis存储数据
                    self.__storager.real_time_data_storage(format_data)
                    self._log.info('success -- 实时数据存入redis！')
