"""
@Date  :2021/6/16/14:47
@Author:zz
@Desc  :投喂数据统计
"""

import time
import json
import threading
import traceback

from event_storage import EventStorage
from utility import Utility
from log import OutPutLog
from apis.get_feeding_advise import GetFeedingAdvise


class FeedingStatistics:
    def __init__(self):
        self._storage = EventStorage()
        self._log = OutPutLog()
        self._run_time = None
        self._feed_id = None

    def str_to_flaot(self, str):
        if str != None:
            return float(str)
        else:
            return None

    # 历史存储主函数
    def run(self):
        self._log.info('[AutoFeeding] - FeedingStatistics module is running!')
        # 风向：c1，风速：c2,流速：c7，流向：c8
        # 1号风机频率：c651，2号风机频率：c655
        # 1号供料器频率：c659，2号供料器频率：c663，3号供料器频率：c667
        # 分配器位置01:c733,分配器位置02:c734,分配器位置03:c735,分配器位置04:c736
        list = ["c1", "c2", "c7", "c8", "c651", "c655", "c659", "c663", "c667", "c733", "c734", "c735", "c736"]
        # 获取配置信息
        while 1:
            time.sleep(5)
            real_data_dict = self._storage.get_real_data(list)

            # 记录投喂过程
            try:
                feeder_frequency = 0
                # 判断供料器
                if float(real_data_dict['c659']) > 1:
                    feeder_frequency = float(real_data_dict['c659'])
                    tank_number = 1
                elif float(real_data_dict['c663']) > 1:
                    feeder_frequency = float(real_data_dict['c663'])
                    tank_number = 2
                elif float(real_data_dict['c667']) > 1:
                    feeder_frequency = float(real_data_dict['c667'])
                    tank_number = 3
                else:
                    self._feed_id = None

                # 判断投饵口
                if int(real_data_dict['c733']) == 1:
                    feeding_port = 1
                elif int(real_data_dict['c734']) == 1:
                    feeding_port = 2
                elif int(real_data_dict['c735']) == 1:
                    feeding_port = 3
                elif int(real_data_dict['c736']) == 1:
                    feeding_port = 4

                # 判断风机
                if float(real_data_dict['c651']) > 0:
                    fen_frequency = float(real_data_dict['c651'])
                elif float(real_data_dict['c655']) > 0:
                    fen_frequency = float(real_data_dict['c655'])

                if feeder_frequency > 1:
                    now_time = time.time()
                    if self._feed_id == None:
                        self._feed_id = 1
                        feed_id = int(now_time)
                    if self._run_time == None:
                        self._run_time = int(now_time)
                    save_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(now_time))
                    wind_speed = self.str_to_flaot(real_data_dict['c2'])
                    wind_direction = self.str_to_flaot(real_data_dict['c1'])
                    flow_speed = self.str_to_flaot(real_data_dict['c7'])
                    flow_direction = self.str_to_flaot(real_data_dict['c8'])

                    sql_get_task_info = "SELECT * FROM touer_set;"
                    task_info = self._storage.execute_sql(sql_get_task_info)
                    task_info = task_info[0]
                    if task_info['feed_mode'] == 1:
                        feeding_mode = "自动"
                    elif task_info['feed_mode'] == 0:
                        feeding_mode = "手动"
                    else:
                        feeding_mode = None
                    breed = task_info['feed_kind']
                    tank_number_info = eval(task_info["tank" + str(tank_number)])
                    bait_name = tank_number_info['name']
                    feed_weight = float(tank_number_info['weight'])
                    feed_diameter = int(tank_number_info['diameter'])
                    shelf_life = tank_number_info['shelf_life']
                    feeding_speed = 1440 / 67 * feeder_frequency / 50 * (8 - 0.5 * feed_diameter) * 1 / 60
                    now_time = int(time.time())
                    feeding_amount = round(feeding_speed * (now_time - self._run_time), 2)
                    self._run_time = now_time
                    sql = "INSERT INTO fishery_data(feed_id, times, wind_speed, wind_direction,\
                           flow_speed, flow_direction, feeding_port, tank_number, feeder_frequency,\
                           fan_frequency, feeding_amount, feeding_mode, breed, bait_name, feed_weight, feed_diameter, shelf_life) \
                           VALUES (%s,'%s',%s,%s,%s,%s,%s,%s,%s,%s,%s,'%s','%s','%s',%s,%s,'%s')" % \
                          (feed_id, save_time, wind_speed, wind_direction, flow_speed, flow_direction, feeding_port, \
                           tank_number, feeder_frequency, fen_frequency, feeding_amount, feeding_mode, breed, bait_name,
                           feed_weight, feed_diameter, shelf_life)

                    self._storage.execute_sql(sql)
                else:
                    self._run_time = None
            except Exception as e:
                print('行号', e.__traceback__.tb_lineno)
                print("get fishery_data [ERROR]:" + str(e))
                self._log.error("get fishery_data [ERROR]:" + str(e))