#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2021/12/8 14:57
@Author: lxc
@LastEditTime: 
@Desctiption: 聚鱼灯的连接器，控制聚鱼灯的开关
"""

import json
import threading
from connector import Connector
from event_storage import EventStorage
from log import OutPutLog
import time
from _datetime import datetime
import modbus_tk.modbus_tcp as modbus_tcp


class GatherFishLightConnector(Connector, threading.Thread):

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self._master = None
        self.__stopped = False
        self._connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self._converter = converter
        self._storager = EventStorage()
        self._name = name
        self.run_time = 0
        self._command = json.loads(self._storager.get_command_info(name[:-3])[0]["command"])
        """
        self._command中的命令
        light_0：关灯
        light_1：开灯
        light_status：查询当前状态
        photosense_0：关光敏
        photosense_1：开光敏

        light_0 [{'res': 0, 'device_id': 1, 'start_addr': 0, 'output_value': 0, 'function_code': 5}, {'res': 0, 'device_id': 1, 'start_addr': 1, 'output_value': 0, 'function_code': 5}]
        light_1 [{'res': 65280, 'device_id': 1, 'start_addr': 0, 'output_value': 65280, 'function_code': 5}, {'res': 65280, 'device_id': 1, 'start_addr': 1, 'output_value': 65280, 'function_code': 5}]
        light_status {'length': 4, 'device_id': 1, 'start_addr': 0, 'function_code': 1}
        photosense_0 [{'res': 65280, 'device_id': 1, 'start_addr': 2, 'output_value': 65280, 'function_code': 5}]
        photosense_1 [{'res': 0, 'device_id': 1, 'start_addr': 2, 'output_value': 0, 'function_code': 5}]
        """

    def open(self):
        self.__stopped = False
        self.start()


    def run(self):
        self._connect()
        self._connected = True

        while True:
            this_time = time.time()
            if this_time - self.run_time > 5:
                self.run_time = this_time

                light_info = self._storager.gather_fish_light_info(self._name[-2:])
                for each_light in light_info:
                    self.check_status(each_light)
                    self.command_polling(each_light)


    def _connect(self):
        try:
            self._master = modbus_tcp.TcpMaster(host=self._ip, port=self._port)
            print(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), "connect success!", self._ip, self._port)
        except Exception as e:
            print(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), "connect failed!", self._ip, self._port, e)
            self._log.error(f"聚鱼灯modbus_tk连接失败！{e}")
            self._connected = False
            self._reconnect()


    def _reconnect(self):
        while True:
            try:
                self._master = modbus_tcp.TcpMaster(host=self._ip, port=self._port)
                print(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), "reconnect success!", self._ip, self._port)
                break
            except Exception as e:
                print(datetime.now().strftime('%Y-%m-%d %H:%M:%S'), "reconnect failed!", self._ip, self._port, e)
                time.sleep(5)


    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self._connected

    def send_command(self, data):

        res = True
        result = True
        try:
            for each_command in data:
                result = self.exec_command(each_command)
                if result is False:
                    res = False
        except Exception as e:
            print(f"[GatherFishLightConnector][send_command] error: {e}")
            res = False

        if res is True:
            res = result

        return res


    def exec_command(self, command):
        """
        0x01: 读线圈寄存器
        0x02: 读离散输入寄存器
        0x03: 读保持寄存器
        0x04: 读输入寄存器
        0x05: 写单个线圈寄存器
        0x06: 写单个保持寄存器
        0x0f: 写多个线圈寄存器
        0x10: 写多个保持寄存器
        #supported modbus functions
        READ_COILS = 1
        READ_DISCRETE_INPUTS = 2
        READ_HOLDING_REGISTERS = 3
        READ_INPUT_REGISTERS = 4
        WRITE_SINGLE_COIL = 5
        WRITE_SINGLE_REGISTER = 6
        READ_EXCEPTION_STATUS = 7
        DIAGNOSTIC = 8
        REPORT_SLAVE_ID = 17
        WRITE_MULTIPLE_COILS = 15
        WRITE_MULTIPLE_REGISTERS = 16
        READ_WRITE_MULTIPLE_REGISTERS = 23
        DEVICE_INFO = 43
        """
        if isinstance(command, str):
            command = json.loads(command)
        device_id = int(command['device_id'])
        function_code = int(command['function_code'])
        start_addr = int(command['start_addr'])

        if function_code in (1, 2, 3, 4):
            # 读寄存器
            try:
                self._master.set_timeout(1)
                self._master.set_verbose(True)
                length = command['length']
                data = self._master.execute(device_id, function_code, start_addr, length)
                return data
            except Exception as e:
                self._log.error(e)
                # print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')} [GatherFishLightConnector] {self._ip} {command}")
                print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')} [GatherFishLightConnector] {self._ip} {e}")
                return False


        elif function_code in (5, 6, 15, 16):
            # 写寄存器
            try:
                self._master.set_timeout(1)
                self._master.set_verbose(True)
                output_value = command['output_value']
                # print("device_id = ", device_id,  function_code, start_addr, output_value)
                data = self._master.execute(device_id, function_code, start_addr, output_value=output_value)
                print("-------------------------------", self._ip, command, 'res=', data)
                res = command["res"]
                result = False
                if function_code == 5:
                    if start_addr == data[0] and res == data[1]:
                        result = True
                time.sleep(1)
                return result
            except Exception as e:
                self._log.error(e)
                # print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')} [GatherFishLightConnector] {self._ip} {command}")
                print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')} [GatherFishLightConnector] {self._ip} {e}")

                return False
        else:
            self._log.debug(f'Unsupported function code.')



    def command_polling(self, data, resend_times=None):
        """
        判断当前模式需要发送的指令
        data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]], 'conn_status': 0}
        control_mode：控制模式，0：光敏控制，1：手动控制，2：定时控制
        status：聚鱼灯的状态，0：关，1开，手动模式需要获取该参数的值
        """
        control_mode = data["control_mode"]
        # 如果是定时模式，在这里控制灯的开关
        if control_mode == 2:
            '''定时模式'''
            light_status = 0
            now_time = datetime.now().strftime("%H:%M:%S")
            definite_time = json.loads(data["definite_time"])
            # 依次判断每个时间段
            for each_time in definite_time:
                each_time = each_time[0].split(",")
                light_open_time = each_time[0]
                light_close_time = each_time[1]
                if light_open_time < light_close_time:
                    if light_open_time <= now_time <= light_close_time:
                        light_status = 1
                else:
                    if now_time >= light_open_time or now_time <= light_close_time:
                        light_status = 1
            light_command = self._command[f"light_{light_status}"]
            for each_command in light_command:
                self.exec_command(each_command)



    # 原版：三种模式逻辑正常判断
    """
    def command_polling(self, data, resend_times=None):
        '''判断当前模式需要发送的指令'''
        # data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]]}
        control_mode = data["control_mode"]     # 0：光敏控制，1：手动控制，2：定时控制
        photosense_status = 0                   # 光敏模式的状态，默认为关闭
        light_command = None

        if control_mode == 0:
            '''光敏模式'''
            photosense_status = 1
        else:
            light_status = 0        # 手动/定时模式时，默认聚鱼灯是关闭状态
            if control_mode == 1:
                '''手动模式'''
                light_status = data["status"]
            elif control_mode == 2:
                '''定时模式'''
                now_time = datetime.now().strftime("%H:%M:%S")
                definite_time = json.loads(data["definite_time"])
                # 依次判断每个时间段
                for each_time in definite_time:
                    each_time = each_time[0].split(",")
                    light_open_time = each_time[0]
                    light_close_time = each_time[1]
                    if light_open_time < light_close_time:
                        if light_open_time <= now_time <= light_close_time:
                            light_status = 1
                    else:
                        if now_time >= light_open_time or now_time <= light_close_time:
                            light_status = 1
            light_command = self._command[f"light_{light_status}"]

        # 开启/关闭光敏模式的指令
        command = self._command[f"photosense_{photosense_status}"]
        self.exec_command(command)
        # 非光敏模式下，控制聚鱼灯开关
        if light_command is not None:
            self.exec_command(light_command)

    # 特殊版：应对经海四号的光敏模式闪烁问题   就固定早7点关闭，晚6点开启
    def command_polling(self, data, resend_times=None):
        '''判断当前模式需要发送的指令'''
        # data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]]}
        control_mode = data["control_mode"]     # 0：光敏控制，1：手动控制，2：定时控制
        photosense_status = 0                   # 光敏模式的状态，默认为关闭
        light_command = None

        if control_mode == 0:
            '''光敏模式'''
            # photosense_status = 1
            photosense_status = 0
            now = datetime.now().strftime("%H")
            if 7 <= int(now) < 18:
                # 关灯
                light_command = self._command["light_0"]
            else:
                # 开灯
                light_command = self._command["light_1"]
        else:
            light_status = 0        # 手动/定时模式时，默认聚鱼灯是关闭状态
            if control_mode == 1:
                '''手动模式'''
                light_status = data["status"]
            elif control_mode == 2:
                '''定时模式'''
                now_time = datetime.now().strftime("%H:%M:%S")
                definite_time = json.loads(data["definite_time"])
                # 依次判断每个时间段
                for each_time in definite_time:
                    each_time = each_time[0].split(",")
                    light_open_time = each_time[0]
                    light_close_time = each_time[1]
                    if light_open_time < light_close_time:
                        if light_open_time <= now_time <= light_close_time:
                            light_status = 1
                    else:
                        if now_time >= light_open_time or now_time <= light_close_time:
                            light_status = 1
            light_command = self._command[f"light_{light_status}"]
            # print("-------------------------------------------", self._ip, light_status, light_command)
        # 开启/关闭光敏模式的指令
        # command = self._command[f"photosense_{photosense_status}"]
        # self.exec_command(command)
        # 非光敏模式下，控制聚鱼灯开关
        if light_command is not None:
            self.exec_command(light_command)

    """



    def check_status(self, data):
        """
        判断当前模式需要发送的指令
        data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]], 'conn_status': 0}
        control_mode：控制模式，0：光敏控制，1：手动控制，2：定时控制
        status：聚鱼灯的状态，0：关，1开，手动模式需要获取该参数的值
        """
        p_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        # 查看当前状态
        device_status_commang = self._command['light_status']     # {'length': 4, 'device_id': 1, 'start_addr': 0, 'function_code': 1}
        # device_status_res = (0, 0, 1, 0), 4路继电器，第1路、第2路表示灯开关（0=关， 1=开），第3路表示光敏模式开关（0=开，1=关）
        device_status_res = self.exec_command(device_status_commang)


        # 对比现在的状态和断电之前的状态
        last_conn_status = data['conn_status']   # 上一次记录到的聚鱼灯的状态
        if device_status_res and last_conn_status == 1:
            # 现在能读到聚鱼灯的状态且上一次读不到：将聚鱼灯的状态恢复至断电之前的状态，更新conn_status的状态
            print(f"{p_time}-[check_status]-{self._ip} {device_status_res} 1、2路表示灯开关(0关 1开) 3路表示光敏模式开关(0开 1关)")
            all_command = []

            control_mode = data["control_mode"]     # 查看历史状态（存在数据库中）
            if control_mode == 0:
                # 恢复至光敏模式
                if device_status_res[2] == 1:
                    # 当前是非光敏模式
                    if device_status_res[0] == device_status_res[1] == 1:
                        # 先关灯
                        all_command.extend(self._command['light_0'])
                        print(f"{p_time}-[check_status]-{self._ip} 先关灯，再设为光敏模式")
                    # 再开光敏
                    all_command.extend(self._command['photosense_1'])
                    print(f"{p_time}-[check_status]-{self._ip} 设为光敏模式")
                else:
                    # 当前是光敏模式，不需要处理
                    print(f"{p_time}-[check_status]-{self._ip} 当前是光敏模式，不需要处理")
                    # pass
            elif control_mode == 1:
                # 手动模式
                if device_status_res[2] == 0:
                    # 先关闭光敏模式
                    all_command.extend(self._command['photosense_0'])
                    print(f"{p_time}-[check_status]-{self._ip} 先关闭光敏模式，再修改灯的状态为{data['status']}")
                if device_status_res[0] == device_status_res[1] != data['status']:
                    # 聚鱼灯改为status的状态
                    all_command.extend(self._command[f"light_{data['status']}"])
                    print(f"{p_time}-[check_status]-{self._ip} 修改灯的状态为{data['status']}")
                else:
                    print(f"{p_time} {self._ip} 状态相同{data['status']}，不需要处理")
            elif control_mode == 2:
                # 定时模式
                if device_status_res[2] == 0:
                    # 先关闭光敏模式
                    all_command.extend(self._command['photosense_0'])
                    print(f"{p_time}-[check_status]-{self._ip} 定时模式:先关闭光敏模式")
                # 判断开关灯在 command_polling 方法中


            # 执行修改状态的指令
            for each_command in all_command:
                self.exec_command(each_command)

            # 更新conn_status的状态
            sql = f"UPDATE gather_fish_light_control SET conn_status=0 WHERE location=\'{self._name[-2:]}\';"
            self._storager.execute_update_sql(sql)
        elif device_status_res is False and last_conn_status == 0:
            # 现在读不到聚鱼灯的状态且上一次能读到：更新conn_status的状态
            sql = f"UPDATE gather_fish_light_control SET conn_status=1 WHERE location=\'{self._name[-2:]}\';"
            self._storager.execute_update_sql(sql)




"""
光敏模式(开启) 发送：01 05 00 02 FF 00 2D FA   接收：01 05 00 02 FF 00 2D FA
光敏模式(关闭) 发送：01 05 00 02 00 00 6C 0A   接收：01 05 00 02 00 00 6C 0A
聚鱼灯开启：01 0F 00 00 00 04 01 01 01 00 00 5E 50
聚鱼灯关闭：01 0F 00 00 00 04 00 00 00 00 00 33 AC

01 ———— 设备地址
05 ———— 写单个线圈寄存器
00 02 ———— 起始寄存器地址
FF 00 ———— 寄存器的值
2D FA ———— CRC校验

01          ———— 设备地址
0F          ———— 功能码
00 00       ———— 起始地址
00 04       ———— 写入线圈个数
01          ———— 写入字节数
01 01 00 00 ———— 写入值
5E 50       ———— CRC校验

开启光敏模式 photosense_1= {"device_id":1, "function_code":5, "start_addr":2, "output_value":0}
关闭光敏模式 photosense_0={"device_id":1, "function_code":5, "start_addr":2, "output_value":65280}
开启聚鱼灯 light_1={"device_id":1, "function_code":15, "start_addr":0, "output_value":[1, 1, 0 ,0]}
关闭聚鱼灯 light_0={"device_id":1, "function_code":15, "start_addr":0, "output_value":[0, 0, 0 ,0]}
"""

"""
4路继电器，第1路、第2路控制灯开关（0=关， 1=开），第3路控制光敏模式开关（0=开，1=关）
开光敏：(0, 0, 0, 0)
关光敏：(0, 0, 1, 0)
非光敏（关灯）：(0, 0, 1, 0)
非光敏（开灯）：(1, 1, 1, 0)
"""