#!/usr/bin/env python
# encoding: utf-8
"""
@time: 2021/5/25 11:14
@desc: 查询电流、电压的历史数据
"""

from abstract_api import AbstractApi
from event_storage import EventStorage
import datetime

class GetHistoricalData(AbstractApi):
    """查询历史数据（电流、电压等)"""
    def operation(self, select_info):
        operate_mysql = EventStorage()
        now_time = datetime.datetime.now()           # 2021-05-26 10:33:13.252703
        today_date = now_time.strftime("%Y-%m-%d")   # 2021-05-26
        device_name = select_info['deviceName']
        table_name = "table_" + select_info['deviceName']
        select_time = select_info['selectTime']

        io_point_dict = {}      # {'c151': 'Ua', 'c152': 'Ub', 'c153': 'Uc', 'c154': 'Ia', 'c155': 'Ib', 'c156': 'Ic', 'c157': 'P', 'c158': 'Q', 'c159': '正向有功电能值', 'c160': '正向无功电能值'}
        sql1 = "SELECT serial_number, io_point_name FROM data_point_tbl WHERE device_name=\'%s\';" % (device_name)
        for each in operate_mysql.execute_sql(sql1):
            io_point_dict["c"+str(each['serial_number'])] = each['io_point_name']
            if each['io_point_name'] == '正向有功电能值':
                c_kWh = "c"+str(each['serial_number'])
            if each['io_point_name'] == '正向无功电能值':
                c_kVarh = "c" + str(each['serial_number'])

        return_dict = {}
        # 查给定日期的历史数据
        sql2 = "SELECT * FROM `%s` WHERE DATE_FORMAT(times, '%%Y-%%m-%%d') = \'%s\';" % (table_name, select_time) # 指定日期的所有数据
        all_data = operate_mysql.execute_sql(sql2)

        if len(all_data) == 0:
            # 查询日期无数据时，返回null
            for k in io_point_dict:
                return_dict.setdefault(io_point_dict[k], []).append(None)
        else:
            for history_data in all_data:
                # history_data = {'id': 1, 'times': datetime.datetime(2021, 5, 25, 11, 1, 1), 'is_send': 0, 'c151': 45.0, 'c152': 78.0, 'c153': 24.0, 'c154': 78.0, 'c155': 54.0, 'c156': 74.0, 'c157': 71.0, 'c158': 32.0, 'c159': 54.0, 'c160': 31.0}
                return_dict.setdefault("times", []).append(history_data['times'])
                for k in history_data:
                    # k = 'c151': 45.0
                    if 'c' in k:
                        io_point = io_point_dict[k]
                        value1 = history_data[k]
                        # 当日耗能实时参数 = 今天的最后一条数据 - 今天的第一条数据
                        # 其他参数 = 今天的最后一条数据
                        if k != c_kWh and k != c_kVarh:
                            value = value1
                        else:
                            sql3 = "SELECT %s, %s FROM `%s` WHERE times>\'%s\' ORDER BY times LIMIT 1;" % (c_kWh, c_kVarh, table_name, select_time)  # 指定日期的第一条数据
                            earliest_data = operate_mysql.execute_sql(sql3)  # {'c159': 1.2, 'c160': 2.4}
                            value = value1 - earliest_data[0][k]
                        return_dict.setdefault(io_point, []).append(value)

        """
        # 查最新的一条数据
        sql4 = "SELECT * FROM `%s` ORDER BY times DESC LIMIT 1;" % (table_name)
        latest_data = operate_mysql.execute_sql(sql4)
        for k in latest_data[0]:
            # k = 'c151': 45.0
            if 'c' in k:
                io_point = "latest_" + io_point_dict[k]
                value1 = latest_data[0][k]
                # 当日耗能实时参数 = 今天的最后一条数据 - 今天的第一条数据
                # 其他参数 = 今天的最后一条数据
                if k != c_kWh and k != c_kVarh:
                    value = value1
                else:
                    sql5 = "SELECT %s FROM `%s` WHERE times>\'%s\' ORDER BY times LIMIT 1;" % (k, table_name, today_date)
                    res_data = operate_mysql.execute_sql(sql5)
                    value = value1 - res_data[0][k]
                return_dict[io_point] = value
        """

        return return_dict
