"""
@Date  :2021/7/7/10:25
@Author:zz
@Desc  :自动启动发动机充电
"""

import time
import json
import threading

from event_storage import EventStorage
from utility import Utility
from log import OutPutLog

class AutoCharge:
    def __init__(self):
        self._storage = EventStorage()
        self._log = OutPutLog()

    # 历史存储主函数
    def run(self):
        self._log.info('[AutoCharge] - AutoFeeding module is running!')

        # c101 蓄电池电压
        # c402 主发运行
        # c411 主发自动模式(0：本地模式，1：遥控模式，2：自动模式)
        # c815 主发备车
        # c423 应发运行
        # c432 应发自动模式(0：本地模式，1：遥控模式，2：自动模式)
        # c817 应发备车
        list = ["c101", "c402", "c411", "c815", "c423", "c432", "c817"]

        # 获取配置信息
        while 1:
            command = None
            real_data_dict = self._storage.get_real_data(list)

            sql_wind_light_configuration = "SELECT * FROM wind_light_configuration;"
            wind_light_configuration = self._storage.execute_sql(sql_wind_light_configuration)

            if wind_light_configuration is not None and len(wind_light_configuration) > 0:
                charge_device = int(wind_light_configuration[0]['recharge_device_set'])                 # 充电设备配置 0:无；1:主发电机；2:应急发电机
                charge_voltage = int(wind_light_configuration[0]['recharge_device_start_voltage'])      # 充电设备启动电压(V)
                battery_voltage = real_data_dict['c101']       # 蓄电池电压

                # 判断电池组电压是否低于设定电压
                if battery_voltage is not None and int(battery_voltage) <= charge_voltage:
                    # 判断是否有发电机正在运行
                    if real_data_dict['c402'] != '1' and real_data_dict['c423'] != '1':

                        if charge_device == 1 and real_data_dict['c402'] is not None and real_data_dict['c411'] is not None and real_data_dict['c815'] is not None:
                            # 启用主发电机
                            fdj_status = int(real_data_dict['c402'])
                            fdj_mode = int(real_data_dict['c411'])
                            fdj_beiche = int(real_data_dict['c815'])
                            if fdj_status == 0 and fdj_mode == 2 and fdj_beiche == 1:
                                command = {"device_id": 1, "start_addr": 3, "output_value": 65280, "function_code": 5, "res": 65280}
                            else:
                                print(f"{time.strftime('%Y-%m-%d %H:%M:%S')} {charge_device}设备（1=主发 2=应发）状态错误。当前模式为{fdj_mode}，备车为{fdj_beiche}")
                                self._log.error(f"{time.strftime('%Y-%m-%d %H:%M:%S')} {charge_device}设备（1=主发 2=应发）状态错误。当前模式为{fdj_mode}，备车为{fdj_beiche}")
                        elif charge_device == 2 and real_data_dict['c423'] is not None and real_data_dict['c432'] is not None and real_data_dict['c817'] is not None:
                            # 启动应急发电机
                            fdj_status = int(real_data_dict['c423'])
                            fdj_mode = int(real_data_dict['c432'])
                            fdj_beiche = int(real_data_dict['c817'])
                            if fdj_status == 0 and fdj_mode == 2 and fdj_beiche == 1:
                                command = {"device_id": 1, "start_addr": 4, "output_value": 65280, "function_code": 5, "res": 65280}
                            else:
                                print(f"{time.strftime('%Y-%m-%d %H:%M:%S')} {charge_device}设备（1=主发 2=应发）状态错误。当前模式为{fdj_mode}，备车为{fdj_beiche}")
                                self._log.error(f"{time.strftime('%Y-%m-%d %H:%M:%S')} {charge_device}设备（1=主发 2=应发）状态错误。当前模式为{fdj_mode}，备车为{fdj_beiche}")
            else:
                print(f"{time.strftime('%Y-%m-%d %H:%M:%S')} [AutoCharge]—参数获取失败")
                self._log.info(f"{time.strftime('%Y-%m-%d %H:%M:%S')} [AutoCharge]—参数获取失败")


            if command:
                charge_connectors = Utility.available_connectors["remote_control"]
                change_res = charge_connectors.send_command([command])
                time.sleep(30)
                if change_res:
                    print(f"{time.strftime('%Y-%m-%d %H:%M:%S')} 当前蓄电池组电压为{battery_voltage}V，低于{charge_voltage}V，启动{charge_device}设备（1=主发 2=应发）成功！")
                else:
                    print(f"{time.strftime('%Y-%m-%d %H:%M:%S')} 当前蓄电池组电压为{battery_voltage}V，低于{charge_voltage}V，启动{charge_device}设备（1=主发 2=应发）失败！")
                    self._log.error(f"{time.strftime('%Y-%m-%d %H:%M:%S')} 当前蓄电池组电压为{battery_voltage}V，低于{charge_voltage}V，启动{charge_device}设备（1=主发 2=应发）失败！")

            time.sleep(10)
